/**
 * @class FSA
 * @brief Replace and Register Aequivalent States, Transfer States to Sparse Array
 * @author Max Hadersbeck
 *
 * defines methods, which are necessary to
 * replace and register aequivalent States and methods
 * to transfer States from an FSA to a Sparse Array
 *
 */

#include<map>
#include<vector>
#include<iostream>
#include<fstream>

extern int Debug;

#include "daciuk_sparse.hpp"


void Daciuk_FSA_Sparse::replace_or_register(CellFSAPtr state) {
		if (Debug >= 3) std::wcout << L"replace_or_register: State=" << state << std::endl;
		CellFSAPtr child = state->last_child();
		if (child == NULL) {
			if (Debug >=1) std::wcout << "No Child" << std::endl;
			return;
		}
		if (child->has_children()) {
			replace_or_register(child);
		}

		if (Debug >= 1)
			std::wcout << L"replace_or_register: find equiv state in Register for " << child << std::endl;
		CellFSAPtr q = find_equiv_state_in_register(child);
		if (q != NULL) {
			if (Debug > 1) std::wcout << L" ++++++++++++++++++///+++++ replace from State:" << state << " Child " << child->getNumber() << L" with: "<< q << std::endl;
			state->replaceState(q);
			if (Debug > 1) std::wcout << L" ------------------///------delete from State:" << state << " Child " << child->getNumber() << std::endl;
			child->deleteState();
		} else {
			if (Debug > 1) std::wcout << L" -------------->add to Register " << child << " Number= " << child->getNumber() << std::endl;
			Register_.push_back(child);
			num_elem_in_reg++;
		}
	}

/**
 * Print Content of Register
 */
void Daciuk_FSA_Sparse::print_register () {
// Hier Print Register implementieren
}

/**
 * Transfer the whole Register into the SPARSE Table
 */
void Daciuk_FSA_Sparse::register_into_sparsetable(CellFSAPtr root,SparseArrayManager& table)
	{
		size_t pos;
		for (int i=0; i < num_elem_in_reg; i++) {
			table.storeState(Register_[i],pos);
			debug(1,"Insert into SPARSE_TABLE: Register[" << i << "]=" << Register_[i] << " Number =" << Register_[i]->getNumber()
					<< "final=" << Register_[i]->isFinal()<< " Position SparseTable= " << pos << std::endl);
			std::wcout << std::endl;
		}
		table.storeState(root,pos);
		debug(1,"Inserted into SPARSE_TABLE: ROOT "<< root << " Number = " << root->getNumber() << "Table_POS=" << pos << std::endl);

	}

/**
 * ****************************************************************
 */
