#ifndef DACIUK_SPARSE_HPP
#define DACIUK_SPARSE_HPP DACIUK_SPARSE_HPP

/**
 * @class FSA
 * @brief Header for Replace and Register and  Transfer States to Sparse Array
 * @author Max Hadersbeck
 *
 * defines Structures and Headers for methods, which are necessary to
 * replace and register aequivalent States and Headers for methods
 * to transfer States from an FSA to a Sparse Array
 *
 */

#include<map>
#include<vector>
#include<iostream>

#include "fsa.hpp"
#include "sparse_array_manager.hpp"



/**
 * ****************************************************************
 * We define a Daciuk Automat with Minimization and transfer to a
 * Sparse Array
 * A Daciuk Automat "is-a" FSA Automat
 */
class Daciuk_FSA_Sparse : public FSA {

	/**
	 * Variables for access from everywhere
	 */
public:

	Daciuk_FSA_Sparse() {
		q0_ = std::make_shared<CellFSA>(0);
		num_elem_in_reg = 0;
	}


	/**
	 * gets the root State in q0
	 */
	CellFSA::CellFSAPtr get_q0() { return q0_; }

	/**
	 * sets the root State in q0
	 */
	void set_q0(CellFSA::CellFSAPtr root ) { q0_=root; }


	/**
	 * Print Content of Register
	 */
	void print_register ();

	/**
	 * Transfer the whole Register into the SPARSE Table
	 */
	void register_into_sparsetable(CellFSAPtr root,SparseArrayManager& table);
//	{
//		size_t pos;
//		for (int i=0; i < num_elem_in_reg_; i++) {
//			table.storeState(Register_[i],pos);
//			debug(1,"Insert into SPARSE_TABLE: Register[" << i << "]=" << Register_[i] << " Number =" << Register_[i]->getNumber()
//					<< "final=" << Register_[i]->isFinal()<< " Position SparseTable= " << pos << std::endl);
//			std::wcout << std::endl;
//		}
//		table.storeState(root,pos);
//		debug(1,"Inserted into SPARSE_TABLE: ROOT "<< root << " Number = " << root->getNumber() << "Table_POS=" << pos << std::endl);
//
//	}

	/**
	 * sets Number of Elements, stored currently in the Register
	 */
	void set_num_elem_in_reg (int num_reg) { num_elem_in_reg=num_reg; }

	/**
	 *
	 */
	CellFSAPtr find_equiv_state_in_register(CellFSAPtr child) {
		bool found_equiv_state=false;
		CellFSA::CellFSAPtr q;

		for (int i = num_elem_in_reg -1 ; i >= 0 && !found_equiv_state; i--) {
			q = Register_[i];
			if (Debug > 1)
				std::wcerr << L"find_equiv_state_in_register compare:  " << i << child << "  with: " << q << std::endl;
			if (equivalent(q,child)) {
				found_equiv_state = true;
			}
		}
		if (found_equiv_state) {
			if (Debug > 1)
				std::wcerr << L"find_equiv_state_in_register equiv_state=" << q->getNumber() << L" " << std::endl;
			return q;
		} else {
			if (Debug > 1)
				std::wcerr << L"find_equiv_state_in_register NO equiv_state" << std::endl;
			return NULL;
		}

	}



	/**
	 * Minimizing Automat during insert!
	 * @param[IN] state
	 */
	void replace_or_register(CellFSA::CellFSAPtr state);


	/**
	 * print automat, starting at address root  to dot File out
	 */
	void toDot(CellFSAPtr root,std::wofstream& out,std::wstring result);

	void print_to_dot (std::wofstream& out,CellFSAPtr cell,
				std::wstring result, std::set<CellFSAPtr> &allready_here_dot);

private:

	/**
	 * stores the root State in q0
	 */
	CellFSA::CellFSAPtr q0_;

	/**
	 * the Register of all States to search for minimize
	 */

	std::vector < CellFSA::CellFSAPtr > Register_;

	/**
	 * Number of Elements, stored currently in the Register
	 */
	int num_elem_in_reg;


};



/**
 * ****************************************************************
 */


#endif /* DACIUK_SPARSE_HPP */
