#ifndef GLOBAL_DACIUK_SPARSE_HPP
#define GLOBAL_DACIUK_SPARSE_HPP GLOBAL_DACIUK_SPARSE_HPP

/**
 * @brief  global Definitions and Macros for FSA and Sparse Table
 * @author Max Hadersbeck, based on Ideas from Florian Fink
 *
 */


#include <iostream>

/**
 * The type of an index in the SparseTable
 */
typedef unsigned int trans_t;

/**
 * The maximum number of chars.
 * The value for this corresponds to the UTF8 Encoding, because all
 * wchars are transformed into utf8 multipyte strings
 * This value is used for computing the hash value of a TmpState.
 * @see cis::StateHash::hash
 * @see cis::TmpState
 */
static const std::size_t MAX_CHARS = 256;

/**
 * The default Cells table size of the SparseTable.
 * @see cis::SparseTable
 */
static const std::size_t CELLS_SIZE = 1000000;


/**
 * Remove ending/leading subsequent whitespaces and punctuations from a token.
 * @param[in][out] str ... the token.
 */
static inline void chomp(std::wstring& str)
{
	std::size_t i;
/**
 * remove leading whitespaces or Punctuation Chars at the end
 */
	i = str.size()-1;
	while (i > 0 && (isspace(str[i]) || iswpunct(str[i])) )
			{
			str.erase(i, 1);
			--i;
			}



/**
 * remove leading whitespaces or Punctuation Chars at the beginning
 */
	i = 0;
	while (i < str.size() && (isspace(str[i]) || iswpunct(str[i])) )
			{
			str.erase(i, 1);
			++i;
			}

	/* remove leading whitespace */
	for (std::size_t i = 0; i < str.size(); i++) {
		if (isspace(str[i]) || iswpunct(str[i]))
			str.erase(i, 1);
		else
			break;
	}

}

#define sdebug(lvl,msg)                              \
        if (Debug >= lvl) std::cerr << "### " << __FILE__         \
        << ": " << __LINE__ << ": "             \
        << __func__ << ": " << msg

#define debug(lvl,msg)                             \
        if (Debug >= lvl) std::wcerr << "### " << __FILE__        \
        << ": " << __LINE__ << ": "             \
        << __func__ << ": " << msg



#endif
