/** @mainpage Acyclic Finite State Automat with SPARSE TABLE and C++,
  Projekt: Daciuk Minimized Finite State Automat and Sparse Table
  @author Max Hadersbeck
  @version 1.0
  @date 12. Jan 2017

  @file main_daciuk_sparse_fsa.cpp

  This the the main program of developing a daciuk "minimal acyclic graph"
  and transfer it into a sparse Table


  Starting:
    ./main_daciuk_sparse_fsa <INFILE>
       reads from the File INFILE all words,
    or
    ./main_daciuk_sparse_fsa
     reads from the file "eingabe.txt" or from the console words



  The program calculates the minimized automat and asks the user for words,
  which are searched in the automaton.
  After entering words, the program develops a automat-dot File
  which can be converted to a .png File with the dot program from the Graphiz Softwarepaket.
  Then the automat is converted into a SPARSE Table
  */

#include<iostream>
#include<fstream>
#include <chrono>

/**
 * Debug is a GLOBAL Variable, used over all Modules of the Project
 * other Modules must define it as extern int Debug;
 * set Debug greater than one produces Debug output:
 * The bigger value you select, the more output you get
 * Debug=0 means no Debug Information
 */
int Debug = 0;

#include "global_daciuk_sparse.hpp"
#include "cellfsa.hpp"
#include "fsa.hpp"
#include "daciuk_sparse.hpp"
#include "process_info.h"
#include <vector>
#include <set>
#include <fstream>
#include <cstdlib>
#include <chrono>

typedef std::set<std::wstring> WSet;



int main( int argc, char* argv[] ) {

	std::setlocale(LC_ALL, "de_DE.UTF8");
	std::locale utf8locale ("de_DE.UTF8");
	Daciuk_FSA_Sparse dacfsa;

	std::wstring word,last_word;
	int num_words;
	std::wstring ausgabe(1048,0);
	std::wstring CurrentSuffix = L"               ";
	std::wstring CommonPrefix = L"                ";
	std::string txt_filename="eingabe.txt";
	std::string dot_filename="automat.dot";

	CellFSA::CellFSAPtr LastState;
	int index_level=0;
	std::wifstream in;
	std::wofstream out(dot_filename);;
	pid_t pid = getpid();
	struct process_info proc_info;
	struct process_info_statm  proc_statm;
	//clock_t begin, end;
	WSet wstrSet;
	WSet::iterator pos;
	bool data_from_file = false;

	//  Initialize a SPARSE TABLE:
	SparseArrayManager spa_tab;

	last_word=L"";
	num_words=0;
	// `std::chrono::system_clock` reflects the actually elapsed time much
	// better than the number of clock ticks, especially when the program runs
	// on multiple threads.
	auto start = std::chrono::system_clock::now();


	std::locale::global(utf8locale);
	/**
	 * decide to read from a File, specified via Commandline or from file eingabe.txt
	 */
	if (argc > 1) {
		std::wcout << "We read from File >>" << argv[1] << std::endl;
        in.open(argv[1]);
	}
    if (!in.is_open()) {
            std::wcerr << "Could not open specified file." << std::endl;
            std::wcout << "We read from File >>" << txt_filename.c_str() << std::endl;
            in.open(txt_filename.c_str());
    }

	in.imbue(utf8locale);
	out.imbue(utf8locale);

	if (in) {
		std::wcout << "We read from the file " <<  std::endl;
		data_from_file = true;
		while (!in.eof()) {
			in >> word;
			chomp(word);
			if (word.size() > 0)  {
				num_words++;
				if (Debug >= 1) std::wcerr << "IN : (" << num_words << ")=#" << word << "#" << std::endl;
				/**
				 * to have the inputdata sorted, we insert them into a set container
				 * later we just have to run through the container from the beginning to the end
				 */
				if (Debug >= 0 && ((num_words % 1000) == 0)) std::wcerr << "IN : (" << num_words << ")=#" << word << "# are read" << std::endl;
				wstrSet.insert(word);
			}
		};

		std::wcout << num_words << " aus der Datei gelesen " << std::endl;

		if (Debug >= 2) {
			for (auto e : wstrSet) {
				std::wcout << " wSET = " << e << std::endl;
			}
		}
	}

	// Test if INPUT from File, otherwise read from wcin
	if (! num_words) {
		do {
			std::wcout << "Eingabe Wort (Enter ^D to stop) >>";
			std::wcin >> word;
			chomp(word);
			if (word.size()>0) wstrSet.insert(word);
		} while (!std::wcin.eof());
	}

// Set data pointer to the first Element of sorted Container
	if (data_from_file) {
		pos=wstrSet.begin();
	}
	num_words=0;

// words into the automata:
	while (pos != wstrSet.end() && num_words <= 5000)
	{
		// get Data from sorted SET
		word = *pos ;
		++pos;

		if (word.size() > 0) {
			num_words++;
			chomp(word);
			last_word = word;
			if (Debug >= 0 && ((num_words % 1000) == 0)) std::wcerr << "Wordnumber(" << num_words << ") with word #" << word << "# is inserted" << std::endl;
			if (Debug >= 1) std::wcerr<<"insert >"<< word << L"<" << std::endl;

			CommonPrefix=dacfsa.common_prefix(dacfsa.get_q0(),word);
			if (CommonPrefix.size() == 0) {
				CurrentSuffix = word;
			}
			else {
				CurrentSuffix= word.substr(CommonPrefix.size());
			}
			LastState=dacfsa.delta(dacfsa.get_q0(),CommonPrefix);
			if (Debug >= 1) {
				std::wcerr<<"daciucFSA.common_prefix: State="<< LastState << " Prefix=" << CommonPrefix << std::endl ;
				std::wcerr<< "daciucFSA.CurrentSuffix=" << CurrentSuffix << std::endl;
				std::wcerr<< "daciucFSA.LastState" << LastState << std::endl;
			}

			if (LastState->has_children()) {
				if (Debug >= 1) std::wcerr << "---->I have children, LastState= " << LastState << " we Try a replace " << std::endl;
				dacfsa.replace_or_register(LastState);

			};
			dacfsa.add_suffix(LastState,CurrentSuffix);
			if (Debug >= 1) dacfsa.print_register();
		}
		if (num_words % 1000 == 0) std::wcout << num_words << " Words are inserted in Automat" << std::endl;
	}

	if (Debug >= 1)
	{
		std::wcerr<<"+++++++++++++++++++++++++++++++replace_or_register ROOT: "<< dacfsa.get_q0() << std::endl;
	}
	dacfsa.replace_or_register(dacfsa.get_q0());
	// Root set to non FINAL
	dacfsa.get_q0()->setFinal(false);

	// Print some resource usage statistics
	auto duration_ms = std::chrono::duration_cast<std::chrono::milliseconds>(
	       std::chrono::system_clock::now() - start);
	std::wcout << "\nMinimization took " << duration_ms.count() << "ms "
	          << "(" << duration_ms.count()/(1000.0*60) << " minutes)" << std::endl;


	std::wcout << "Number of words: " << num_words << std::endl;
	get_process_info(pid, &proc_info);
	get_process_info_statm(pid, &proc_statm);
	std::wcout << "Virtual size: " << proc_info.vsize/1024 << " kByte" << std::endl;
	std::wcout << "Resident size: " << proc_info.rss*4 << " kByte" << std::endl;

	do {
		std::wcout << "Enter Word to search (exit with ^D)>>";
		word.clear();
		word=L"";
		std::wcin >> word;
		chomp(word);
	    std::wcout<<"LOOKUP TEST: =>"<< word << "<=" << std::endl;
		if (word.size()>0) {
		if (dacfsa.lookup(dacfsa.get_q0(),word))
		    {
		     std::wcout<<"=>" << word << "<= is found: " << std::endl;
		    }
		  else
		    {
		     std::wcout<<"=>" << word << "<= is not found " << std::endl;
		    }
		}
	} while (word.size() > 0);


	if (Debug >= 1) {
		std::wcerr << std::endl << "Print TRIE" << std::endl << std::endl;
		dacfsa.print(dacfsa.get_q0(),L"",index_level,ausgabe);
	}
	std::wcerr << std::endl << "Print to DOT" << std::endl << std::endl;
	dacfsa.toDot(dacfsa.get_q0(),out,ausgabe);
	out.close();

	// Print content of the registers:
	if (Debug >= 0) {
		std::wcout << std::endl << std::endl;
		dacfsa.print_register();
		std::wcout<< "Root -->";
		dacfsa.get_q0()->print();
		std::wcout << std::endl;
	}

	/** Now we start with the SPARSE Table
	 * **************************************************
	 */


	Debug = 0;
	// Initialize sparse Table
	spa_tab.initConstruction();
	// transfer register into Sparse Table
	dacfsa.register_into_sparsetable(dacfsa.get_q0(),spa_tab);
	// close Sparse Table
	spa_tab.finishConstruction();

	std::wcin.ignore();
	std::wcout << "Lookup Sparse Table !" << std::endl;
	do {
		std::wcout << "Eingabe Wort (Enter ^D to stop) >>";
		std::wcin.clear();
		word=L"";
		std::wcin >> word;
		chomp(word);
		if (word.size() > 0) {
			if (spa_tab.contains(word)) {
				std::wcout << "#" << word << "#" << " ist in der Sparsetable " << std::endl;
			}
			else {
				std::wcout << "#" << word << "#" << "ist NICHT in der Sparsetable " << std::endl;

			}
		}

	} while (word.size()>0);

	std::wcout << std::endl;



	return 0;
}
