/**
 * @class SparseArray
 * @brief Methods for a working with SparseArrays
 * @author Max Hadersbeck
 *
 * defines methods, which are necessary to
 * work with sparse Arrays
 *
 */

#include "sparse_array.hpp"
#include <iostream>
#include <cstdlib>
#include <cstring>

/*****************************************************************************/
SparseArray::SparseArray(): size_(0), array_(NULL) {}

/*****************************************************************************/
SparseArray::SparseArray(std::size_t n): size_(n), array_(NULL)
{
	array_ = static_cast<SparseCell *>(calloc(sizeof(SparseCell), size_));
}

/*****************************************************************************/
SparseArray::SparseArray(const SparseArray& rhs)
{
	copy(rhs);
}
/*****************************************************************************/
SparseArray& SparseArray::operator=(const SparseArray& rhs)
{
	copy(rhs);
	return *this;
}

/*****************************************************************************/
SparseArray::~SparseArray()
{
	free(array_);
}

/*****************************************************************************/
SparseCell& SparseArray::operator[](std::size_t index)
{
	return array_[index];
}


/*****************************************************************************/
SparseCell& SparseArray::at(std::size_t index)
{
	if (index < size_)
		return array_[index];
	else
		throw std::out_of_range("Cells::at");
}

/*****************************************************************************/
const SparseCell& SparseArray::at(std::size_t index) const
        throw(std::out_of_range)
{
       if (index < size_)
                return array_[index];
        else
                throw std::out_of_range("cis::Cells::at");
}


/*****************************************************************************/
void SparseArray::setSize(std::size_t n)
{
	array_ = static_cast<SparseCell *>(realloc(array_,
			(sizeof(SparseCell) * n)));
	if (n > size_)
		memset(&array_[size_], 0, (n - size_) * sizeof(SparseCell));
	size_ = n;
}

/*****************************************************************************/
std::size_t SparseArray::getSize()
{
	return size_;
}

/*****************************************************************************/
std::size_t SparseArray::getSize() const
{
	return size_;
}

/*****************************************************************************/
void SparseArray::copy(const SparseArray& rhs)
{
        SparseCell *tmp = array_;
	size_ = rhs.size_;
	array_ = static_cast<SparseCell *>(malloc(sizeof(SparseCell) * size_));
	memcpy(array_, rhs.array_, (sizeof(SparseCell) * size_));
	free(tmp);
}
/*****************************************************************************/
std::wostream& operator<<(std::wostream& out, SparseArray& c)
{
	for (std::size_t i = 0; i < c.getSize(); ++i)
		out << c.at(i) << std::endl;
	return out;
}

/*****************************************************************************/
std::wostream& operator<<(std::wostream& out, const SparseArray& c)
{
	for (std::size_t i = 0; i < c.getSize(); ++i)
		out << c.at(i) << std::endl;
	return out;
}
