/**
 * @class SparseArrayManager
 * @brief implementing the Sparse Array
 * @author Max Hadersbeck, based on Ideas from Florian Fink
 * *
 * defines the structure and methods, which are necessary to
 * fill, extend and erase Elements in a Sparse Array
 *
 */


#include "sparse_array_manager.hpp"

#include "cellfsa.hpp"

extern int Debug;

#include "global_daciuk_sparse.hpp"

/*****************************************************************************/
SparseArrayManager::SparseArrayManager(): table_(), free_(), init_(), last_() {}


/*****************************************************************************/
int SparseArrayManager::initialState()
{
	return init_;
}

/*****************************************************************************/
void SparseArrayManager::storeState(const  CellFSAPtr s, size_t &slot)
{
	TransMap_t::const_iterator i = s->get_transitions().begin();
	wchar_t c;
	if (Debug >= 1) s->print();
	int target_index;
	int transition_offset;

	if (i == s->get_transitions().end() ) {
		c =  0;
	}
	else c=i->first;

	debug(1,"Store Transition Character = #" << c << "#" << std::endl;)
	/*
	 * Store the Sparse_table index of the Cell
	 */
	slot = findSlot(*s);
	debug(1,"#+                                       ------>FREE CELL found, SparseTable [" << slot << "]" << std::endl);

	states_index_table_[s->getNumber()] = slot;
	table_[slot].type = SparseCell::State;
	if (s->isFinal()) {
		debug (0,"s is final" << std::endl);
	}
	table_[slot].final = s->isFinal();
	table_[slot].offset = c;
	debug(2,"##*Cell Transition Offset = "<< (int) c << std::endl);

	/* store all transitions */
	while (i != s->get_transitions().end()) {
		c = i->first;
		table_[slot + c].type = SparseCell::Trans;
		table_[slot + c].label = c;
		target_index = states_index_table_[i->second->getNumber()];
		debug (0, "#*connect " << c << " to: State=" << i->second->getNumber() <<
				" -> index_table ="<< target_index << std::endl);
		table_[slot + c].target = target_index;

		++i;
		if (i == s->get_transitions().end()) transition_offset = 0;
		else
			transition_offset = i->first - c;
		debug(2,"##*Transition Offset = "<< (int) transition_offset << std::endl);
		table_[slot + c].offset = transition_offset;
	}

	/* find next free position within the window */
	if ( slot > SparseArrayManager::WINDOW_SIZE) {
		debug(2,"##*extent free slot= "<< slot << std::endl);
		free_ = slot - WINDOW_SIZE;
	}
	while (table_[free_].type != SparseCell::Empty)
		++free_;
	last_ = slot;
}
/*****************************************************************************/
int SparseArrayManager::findSlot(const CellFSA& s)
{
//    Hier FIND SLOT Implementieren
	return 0;
}



/*****************************************************************************/
void SparseArrayManager::initConstruction()
{
	table_ = SparseArray(CELLS_SIZE);
	states_index_table_ = (int *) malloc(CELLS_SIZE);
	free_ = 1;
	last_ = 0;
	init_ = 0;
}

/*****************************************************************************/
void SparseArrayManager::finishConstruction()
{
	init_ = last_;
	last_ = 0;
	free_ = 0;
	debug(1,"Finish Sparse Table! init_= " << init_ << std::endl);
}

/*****************************************************************************/
int SparseArrayManager::delta(int s, wchar_t c) const
{
	if ((table_.at(s + c).type == SparseCell::Trans) &&
			(table_.at(s + c).label == c))
		return table_.at(s + c).target;
	else
		return 0;
}
/*****************************************************************************/
bool SparseArrayManager::contains(const std::wstring& key) const
{
	int s = initialState();
	const wchar_t *wstr = key.c_str();
	debug(1,"TEST contains : " << key << std::endl);
//	wchar_t wc;
	for (size_t i=0; s && i < key.size(); ++i) {
		s = delta(s, wstr[i]);
		debug (1,"TEST Character : " << wstr[i] << " goto Stateindex -->" << s << std::endl);
	}
	if (isFinal(s)) {
		std::wcout << "Stateindex = " << s << " YES! It is  present" << std::endl;
		return true;
	}
	else {
		std::wcout << "Stateindex = " << s << "NO! It is not present" << std::endl;
		return false;
	}
}


/*****************************************************************************/
int SparseArrayManager::initialState() const
{
	return init_;
}

/*****************************************************************************/
bool SparseArrayManager::isFinal(int s)
{
	std::wcout << " TABLE s=" << s << "Type  t=" << table_[s].type << std::endl;
    if (table_[s].type == SparseCell::State) {
    	if (table_[s].final) {
    	debug (1," State s=" << s << "is Final");
    	return true;
    	}
    }
	debug (1," State s=" << s << "is NOT Final");

	return false;}

/*****************************************************************************/
bool SparseArrayManager::isFinal(int s) const
{
	std::wcout << " TABLE s=" << s << std::endl;
    if (table_.at(s).type == SparseCell::State) {
    	if (table_.at(s).final) {
    	debug (0," State s=" << s << "is Final");
    	return true;
    	}
    }
	debug (0," State s=" << s << "is NOT Final");

	return false;
}

/*****************************************************************************/

void SparseArrayManager::setSizeIndexTable(int n)
{
	states_index_table_ = static_cast<int *>(realloc(states_index_table_,
			(sizeof(int) * n)));
}


/*****************************************************************************/
wchar_t SparseArrayManager::getLabel(int s)
{
	return table_.at(s).type == SparseCell::Trans ?
			table_.at(s).label : 0;
}

/*****************************************************************************/
int SparseArrayManager::getOffset(int s)
{
	return table_.at(s).offset;
}

/*****************************************************************************/
std::size_t SparseArrayManager::getSize()
{
	return table_.getSize();
}

