#ifndef CELLSPARSEHPP
#define CELLSPARSE_HPP

/**
 * @class SparseCell
 * @brief defines Structure and Methods for a Sparse Cell
 * @author Max Hadersbeck, based on Ideas from Florian Fink
 * *
 * Headerfile, which defines the stucture of a Sparse Cell and an
 * output Method
 *
 */

#include <iostream>
#include "global_daciuk_sparse.hpp"

/**
 * A cell is either a transition, a state or empty.
 * Each cell stores its type and an offset value.
 *   The offset points to the first transition if it is a state
 *   or to the next transition if it is a transition.
 *   If offset is 0, there are no more transitions.
 *
 * States only store if they are final or not.
 * Transitions store their labels and their target state number.
 * @author Florian Fink/Max Hadersbeck
 */
class SparseCell {
public:
	/**
	 * The type of the cell.
	 */
	enum CellType {
		Empty = 0, ///< Marks an empty cell.
		State, ///< Marks that the cell is a state.
		Trans, ///< Marks that the cell is a transition.
	};

	CellType type;///< The type of the cell.
	int offset; ///< The offset of the next transition.
	union {
		struct {
			int target;    ///< The target state of the transition.
			wchar_t label; ///< The label of a transition.
		};
		struct {
			bool final; ///< The final information of a state.
		};
	};
};
/**
 * Write a cell to a stream.
 * @param out The open stream.
 * @param c The cell to write.
 * @return The stream.
 */
std::wostream& operator<<(std::wostream& out, const SparseCell& c);

#endif /* CELLSPARSE_HPP */
